import { firebase, } from './firebase.methods';
export class FirestoreQueryBuilder {
    constructor() {
        this.queryConfig = {
            constrains: [],
        };
    }
    withDb(db) {
        this.db = db;
        return this;
    }
    withApiRequest(apiRequest) {
        this.apiRequest = apiRequest;
        return this;
    }
    build() {
        const { argsNodeName, argsApiVerb, argsNodeType, resourceId } = this.apiRequest; // prettier-ignore
        let ref;
        switch (argsNodeType) {
            case 'SINGLE_NODE':
                this.queryConfig.constrains.push(firebase.where('id', '==', argsNodeName));
                ref = firebase.collection(this.db, 'singles');
                return firebase.query.apply(this, [
                    ref,
                    ...this.queryConfig.constrains,
                ]);
            case 'GROUPED_LIST_NODE':
                if (argsApiVerb === 'getItems') {
                    ref = firebase.collection(this.db, argsNodeName, resourceId, 'items');
                }
                else {
                    ref = firebase.collection(this.db, argsNodeName);
                }
                return firebase.query.apply(this, [
                    ref,
                    ...this.queryConfig.constrains,
                ]);
            case 'COLLECTION_NODE':
            default:
                ref = firebase.collection(this.db, argsNodeName);
                return firebase.query.apply(this, [
                    ref,
                    ...this.queryConfig.constrains,
                ]);
        }
    }
}
