import { FirestoreQueryBuilder } from './FirestoreQueryBuilder';
import { guid4 } from 'shared-base';
import { itemsToObject, ts } from 'shared-base';
import { NodeType } from 'redux-store-generator';
import { ResponseBuilder } from '../_base/ResponseBuilder';
import { firebase } from './firebase.methods';
// import { firebase, FirebaseApp, Firestore } from 'firestore-local';
export class FirestoreAdapter {
    constructor(app) {
        this.GET = async (request, response) => {
            const firebaseQuery = new FirestoreQueryBuilder()
                .withApiRequest(request)
                .withDb(this.db)
                .build();
            const snapshot = await firebase.getDocs(firebaseQuery);
            const data = snapshot.docs.map((doc) => doc.data());
            response.withData(data);
            return {
                data,
            };
        };
        this.POST_main = async (request, response) => {
            const { argsNodeName, argsParams: data = {} } = request;
            let snapshotGet, ref, itemToAdd;
            itemToAdd = this.withDates(data, true, true);
            const { id } = data;
            if (id) {
                ref = firebase.doc(this.db, argsNodeName, id);
                await firebase.setDoc(ref, itemToAdd);
                snapshotGet = await firebase.getDoc(ref);
            }
            else {
                ref = firebase.collection(this.db, argsNodeName);
                const snapshotAdd = await firebase.addDoc(ref, itemToAdd);
                ref = firebase.doc(this.db, argsNodeName, snapshotAdd.id);
                await firebase.setDoc(ref, {
                    id: snapshotAdd.id,
                }, {
                    merge: true,
                });
                snapshotGet = await firebase.getDoc(ref);
            }
            const getData = snapshotGet.data();
            response.withData(getData);
            return {
                data: getData,
            };
        };
        this.POST_sub = async (request, response) => {
            const { argsNodeName, argsParams: data = {}, resourceId: id } = request;
            let snapshotGet, ref, itemToAdd;
            const { id: itemId = guid4() } = data.items[0];
            itemToAdd = this.withDates(data.items[0], true, true);
            if (itemId) {
                ref = firebase.doc(this.db, argsNodeName, id, 'items', itemId);
                await firebase.setDoc(ref, itemToAdd);
                snapshotGet = await firebase.getDoc(ref);
            }
            else {
                ref = firebase.collection(this.db, argsNodeName, id, 'items');
                const snapshotAdd = await firebase.addDoc(ref, itemToAdd);
                ref = firebase.doc(this.db, argsNodeName, id, 'items', snapshotAdd.id);
                snapshotGet = await firebase.getDoc(ref);
            }
            const getData = snapshotGet.data();
            response.withData(getData);
            return {
                data: getData,
            };
        };
        this.POST_withItems = async (request, response) => {
            const { argsNodeName, argsParams: data = {}, resourceId } = request;
            let snapshotGet, ref, itemToAdd, listItemToAdd;
            const id = resourceId || data.id || guid4();
            const { items = [] } = data;
            itemToAdd = this.withDates(data, true, true);
            delete itemToAdd['items'];
            ref = firebase.doc(this.db, argsNodeName, id);
            await firebase.setDoc(ref, itemToAdd);
            const batch = firebase.writeBatch(this.db);
            items.forEach((item) => {
                const { id: itemId = guid4() } = item;
                listItemToAdd = this.withDates(item, true, true);
                const ref = firebase.doc(this.db, argsNodeName, id, 'items', itemId);
                batch.set(ref, listItemToAdd);
            });
            await batch.commit();
            snapshotGet = await firebase.getDoc(ref);
            const getData = snapshotGet.data();
            response.withData(getData);
            return {
                data: getData,
            };
        };
        this.POST = async (request, response) => {
            const { argsApiVerb, argsParams: data = {} } = request;
            if (argsApiVerb === 'pushItem') {
                return this.POST_sub(request, response);
            }
            else if (data.items) {
                return this.POST_withItems(request, response);
            }
            else {
                return this.POST_main(request, response);
            }
        };
        this.PATCH = async (request, response) => {
            let { argsNodeName, argsApiVerb, argsParams, resourceId: id, resourceItemId: itemId, } = request;
            const data = { ...argsParams };
            let snapshotGet;
            let nodeName = argsNodeName;
            if (request.argsNodeType === NodeType.SINGLE_NODE) {
                nodeName = 'singles';
                id = argsNodeName;
            }
            if (!id) {
                return Promise.resolve(false);
            }
            let ref;
            if (argsApiVerb === 'patchItem') {
                ref = firebase.doc(this.db, argsNodeName, id, 'items', itemId);
            }
            else {
                ref = firebase.doc(this.db, nodeName, id);
            }
            await firebase.setDoc(ref, this.withDates(data, false, true), {
                merge: true,
            });
            snapshotGet = await firebase.getDoc(ref);
            const getData = snapshotGet.data();
            response.withData(getData);
            return {
                data: getData,
            };
        };
        this.PUT = (request, _response) => {
            const { argsParams: data } = request;
            return Promise.resolve();
        };
        this.DELETE_withItems = async (request, _response) => {
            const { argsNodeName, argsParams: data = {}, resourceId: id } = request;
            let ref;
            if (!id) {
                return Promise.resolve(false);
            }
            ref = firebase.collection(this.db, argsNodeName, id, 'items');
            const snapshot = await firebase.getDocs(ref);
            const items = snapshot.docs.map((doc) => doc.data());
            const itemIds = items.map((i) => i.id);
            const batch = firebase.writeBatch(this.db);
            ref = firebase.doc(this.db, argsNodeName, id);
            batch.delete(ref);
            itemIds.forEach((itemId) => {
                ref = firebase.doc(this.db, argsNodeName, id, 'items', itemId);
                batch.delete(ref);
            });
            await batch.commit();
            return {};
        };
        this.DELETE = async (request, _response) => {
            const { argsApiVerb, argsNodeType, argsNodeName, resourceId: id, resourceItemId: itemId, } = request;
            if (!id) {
                return Promise.resolve(false);
            }
            let ref = firebase.doc(this.db, argsNodeName, id);
            if (argsNodeType === 'GROUPED_LIST_NODE') {
                if (argsApiVerb === 'deleteItem') {
                    ref = firebase.doc(this.db, argsNodeName, id, 'items', itemId);
                }
                else {
                    return this.DELETE_withItems(request, _response);
                }
            }
            await firebase.deleteDoc(ref);
        };
        this.fireRequest = (request) => {
            const response = new ResponseBuilder(request);
            const apiMethod = this[request.argsMethod];
            if (typeof apiMethod !== 'function') {
                Promise.resolve(response);
            }
            return new Promise((resolve) => {
                apiMethod(request, response)
                    .then((_res) => {
                    const res = _res;
                    response.withIsSuccess(true).withFirestoreResponse(res);
                    const data = this.parseReturnedData(request, res);
                    response.withData(data);
                    resolve(response.build());
                })
                    .catch(function (error) {
                    response
                        .withErrorType('javascript')
                        .withErrorMessage(error.message);
                    resolve(response.build());
                });
            });
        };
        this.db = firebase.getFirestore(app);
    }
    _logRequest(request) {
        console.log(JSON.stringify(request, null, 4));
    }
    parseReturnedData(request, res) {
        const { data } = res;
        if (request.argsMethod === 'GET') {
            switch (request.argsNodeType) {
                case NodeType.SINGLE_NODE:
                    return data[0];
                case NodeType.COLLECTION_NODE:
                    return itemsToObject(data, request.argsParams?.page ?? 1, request.argsParams?.limit);
                case NodeType.GROUPED_LIST_NODE:
                    if (request.argsApiVerb === 'getItems') {
                        return data;
                    }
                    return itemsToObject(data, request.argsParams?.page ?? 1, request.argsParams?.limit);
                default:
                    return data;
            }
        }
        else {
            return data;
        }
    }
    withDates(data, withCreatedDate, withModifiedDate) {
        let output = { ...data };
        if (withCreatedDate) {
            output = { ...output, _createdDate: ts() };
        }
        if (withModifiedDate) {
            output = { ...output, _modifiedDate: ts() };
        }
        return output;
    }
}
