import { guid4, getJson, setJson, isEmpty } from 'shared-base';
import { ts } from 'shared-base';
import { NodeType } from 'redux-store-generator';
import { ResponseBuilder } from '../_base/ResponseBuilder';
import { LocalStorageQueryBuilder, } from './LocalStorageQueryBuilder';
import { deleteByPredicate, deleteXPath, getXPath, initStorage, KEY, patchXPath, setXPath, } from './localStorage.methods';
export class LocalStorageAdapter {
    constructor(config) {
        this.config = config;
        this.init = async () => {
            const { on, url } = this.config;
            if (!on) {
                return;
            }
            let data = getJson(KEY);
            if (!data || isEmpty(data)) {
                const response = await fetch(url).then((res) => res.json());
                setJson(KEY, response.data ?? {});
            }
        };
        this.GET = async (request, response) => {
            const storageXPath = new LocalStorageQueryBuilder()
                .withApiRequest(request)
                .build();
            const data = getXPath(storageXPath);
            response.withData(data);
            return {
                data,
            };
        };
        this.POST_main = async (request, response) => {
            const { argsNodeName, argsParams: data = {} } = request;
            let getData, itemToAdd;
            itemToAdd = this.withDates(data, true, true);
            const { id = guid4() } = data;
            const storageXPath = {
                key: argsNodeName,
                xpath: id,
            };
            patchXPath(storageXPath, { ...data, id });
            getData = getXPath(storageXPath);
            response.withData(getData);
            return {
                data: getData,
            };
        };
        this.POST_sub = async (request, response) => {
            const { argsNodeName, argsParams: data = {}, resourceId: id } = request;
            let getData, itemToAdd;
            const { id: itemId = guid4() } = data.items[0];
            itemToAdd = this.withDates(data.items[0], true, true);
            const storageXPath = {
                key: argsNodeName + 'Items',
                xpath: itemId,
            };
            setXPath(storageXPath, data);
            getData = getXPath(storageXPath);
            response.withData(getData);
            return {
                data: getData,
            };
        };
        this.POST_withItems = async (request, response) => {
            const { argsNodeName, argsParams: data = {}, resourceId } = request;
            let getData, itemToAdd, listItemToAdd;
            const id = resourceId || data.id || guid4();
            const { items = [] } = data;
            itemToAdd = this.withDates(data, true, true);
            delete itemToAdd['items'];
            const storageXPath = {
                key: argsNodeName,
                xpath: id,
            };
            setXPath(storageXPath, itemToAdd);
            getData = getXPath(storageXPath);
            items.forEach((item) => {
                const { id: itemId = guid4() } = item;
                listItemToAdd = this.withDates(item, true, true);
                listItemToAdd.parentId = id;
                const itemStorageXPath = {
                    key: argsNodeName + 'Items',
                    xpath: itemId,
                };
                setXPath(itemStorageXPath, listItemToAdd);
            });
            response.withData(getData);
            return {
                data: getData,
            };
        };
        this.POST = async (request, response) => {
            const { argsApiVerb, argsParams: data = {} } = request;
            if (argsApiVerb === 'pushItem') {
                return this.POST_sub(request, response);
            }
            else if (data.items) {
                return this.POST_withItems(request, response);
            }
            else {
                return this.POST_main(request, response);
            }
        };
        this.PATCH = async (request, response) => {
            let { argsNodeName, argsApiVerb, argsParams, resourceId: id, resourceItemId: itemId, } = request;
            const data = { ...argsParams };
            let nodeName = argsNodeName;
            if (request.argsNodeType === NodeType.SINGLE_NODE) {
                nodeName = 'singles';
                id = argsNodeName;
            }
            if (!id) {
                return Promise.resolve(false);
            }
            let storageXPath;
            if (argsApiVerb === 'patchItem') {
                storageXPath = {
                    key: nodeName + 'Items',
                    xpath: itemId,
                };
            }
            else {
                storageXPath = {
                    key: nodeName,
                    xpath: id,
                };
            }
            patchXPath(storageXPath, data);
            const getData = getXPath(storageXPath);
            response.withData(getData);
            return {
                data: getData,
            };
        };
        this.PUT = (request, _response) => {
            const { argsParams: data } = request;
            return Promise.resolve();
        };
        this.DELETE_withItems = async (request, _response) => {
            const { argsNodeName, argsParams: data = {}, resourceId: id } = request;
            let ref;
            if (!id) {
                return Promise.resolve(false);
            }
            let storageXPath = {
                key: argsNodeName,
                xpath: id,
            };
            deleteXPath(storageXPath);
            deleteByPredicate(argsNodeName + 'Items', (item) => item.parentId === id);
            return {};
        };
        this.DELETE = async (request, _response) => {
            const { argsApiVerb, argsNodeType, argsNodeName, resourceId: id, resourceItemId: itemId, } = request;
            if (!id) {
                return Promise.resolve(false);
            }
            let storageXPath = {
                key: argsNodeName,
                xpath: id,
            };
            if (argsNodeType === 'GROUPED_LIST_NODE') {
                if (argsApiVerb === 'deleteItem') {
                    storageXPath = {
                        key: argsNodeName + 'Items',
                        xpath: itemId,
                    };
                }
                else {
                    return this.DELETE_withItems(request, _response);
                }
            }
            deleteXPath(storageXPath);
        };
        this.fireRequest = (request) => {
            const response = new ResponseBuilder(request);
            const apiMethod = this[request.argsMethod];
            if (typeof apiMethod !== 'function') {
                Promise.resolve(response);
            }
            return new Promise((resolve) => {
                apiMethod(request, response)
                    .then((_res) => {
                    const res = _res;
                    response.withIsSuccess(true);
                    const data = this.parseReturnedData(request, res);
                    response.withData(data);
                    resolve(response.build());
                })
                    .catch(function (error) {
                    response
                        .withErrorType('javascript')
                        .withErrorMessage(error.message);
                    resolve(response.build());
                });
            });
        };
        initStorage({
            getJson,
            setJson,
        });
    }
    _logRequest(request) {
        console.log(JSON.stringify(request, null, 4));
    }
    parseReturnedData(request, res) {
        const { data } = res;
        if (request.argsMethod === 'GET') {
            switch (request.argsNodeType) {
                case NodeType.SINGLE_NODE:
                    return data;
                case NodeType.COLLECTION_NODE:
                    return data;
                case NodeType.GROUPED_LIST_NODE:
                    if (request.argsApiVerb === 'getItems') {
                        return data;
                    }
                    return data;
                default:
                    return data;
            }
        }
        else {
            return data;
        }
    }
    withDates(data, withCreatedDate, withModifiedDate) {
        let output = { ...data };
        if (withCreatedDate) {
            output = { ...output, _createdDate: ts() };
        }
        if (withModifiedDate) {
            output = { ...output, _modifiedDate: ts() };
        }
        return output;
    }
}
