import { get, set } from 'shared-base';
let storage = {
    getJson: () => null,
    setJson: () => { },
};
export const KEY = 'LOCAL_DATA_KEY';
export const initStorage = (value) => {
    storage = value;
};
export const getXPath = (storageXPath) => {
    const allData = storage.getJson(KEY) ?? {};
    const arr = storageXPathToPath(storageXPath);
    return get(allData, arr);
};
export const setXPath = (storageXPath, value) => {
    const allData = storage.getJson(KEY) ?? {};
    const arr = storageXPathToPath(storageXPath);
    set(allData, arr, value);
    storage.setJson(KEY, allData);
};
export const patchXPath = (storageXPath, change) => {
    const data = getXPath(storageXPath);
    const newData = {
        ...data,
        ...change,
    };
    setXPath(storageXPath, newData);
};
export const deleteXPath = (storageXPath) => {
    setXPath(storageXPath, undefined);
};
export const getByPredicate = (key, predicate) => {
    const allData = storage.getJson(KEY) || {};
    const nodeData = allData[key] || {};
    return Object.values(nodeData).filter(predicate);
};
export const deleteByPredicate = (key, predicate) => {
    const allData = storage.getJson(KEY) || {};
    const nodeData = allData[key] || {};
    const items = getByPredicate(key, predicate);
    const itemIds = items.map((item) => item.id);
    itemIds.forEach((id) => delete nodeData[id]);
    storage.setJson(KEY, allData);
};
export const storageXPathToPath = (storageXPath) => {
    const { key, xpath } = storageXPath;
    const arr = [key];
    if (xpath) {
        arr.push(xpath);
    }
    return arr;
};
