import { isEmpty, toArray } from 'shared-base';
import { rules } from './GetRulesBuilder';
export class GetRequestBuilder {
    constructor() {
        this.request = {
            path: '',
            headers: {},
            data: {},
            params: [],
        };
        this.rules = rules;
        this.getParams = {};
    }
    withPath(path) {
        this.request.path = path;
        return this;
    }
    withRules(rules) {
        this.rules = rules;
        return this;
    }
    withGetParams(getParams) {
        this.getParams = getParams ?? {};
        return this;
    }
    withMergedGetParamsFilters(filterOrFilters) {
        this.getParams.filter = [
            ...toArray(this.getParams.filter ?? []),
            ...toArray(filterOrFilters),
        ].filter((i) => i);
        return this;
    }
    paramsToQueryStrings() { }
    validateParams(request) {
        request.isInvalid = false;
        request.invalidMessage = '';
    }
    build() {
        let output = { ...this.request };
        this.validateParams(output);
        if (output.isInvalid) {
            return output;
        }
        output = this.rules.base(output, this.getParams);
        const { q, orderBy, filter, page, limit, slice, ...rest } = this.getParams;
        if (q) {
            output = this.rules.fullTextSearch(output, this.getParams);
        }
        if (orderBy) {
            output = this.rules.sort(output, this.getParams);
        }
        if (filter) {
            output = this.rules.filter(output, this.getParams);
        }
        if (page || limit) {
            output = this.rules.paginate(output, this.getParams);
        }
        if (slice) {
            output = this.rules.slice(output, this.getParams);
        }
        if (rest) {
            output = this.rules.rest(output, rest);
        }
        const { params } = output;
        // params
        if (!isEmpty(params)) {
            const qs = params
                .reduce((output, fieldValue) => {
                output.push(`${fieldValue.field}=${encodeURI(fieldValue.value)}`);
                return output;
            }, [])
                .join('&');
            output.path += `?${qs}`;
        }
        return output;
    }
}
/*
export type Order = 'asc' | 'desc';

export type Relation = '==' | '<=' | '<' | '>' | '>=' | '!=' | '//g';

export type Filter = {
    field: string;
    relation: Relation;
    value: string | number | boolean;
};

export type OrderBy = {
    field: string;
    order?: Order;
};

export type Q = {
    field: string;
    value: any;
};

export type Slice = {
    start?: number;
    end?: number;
};

export type GetParams = Partial<{
    q: Q;
    orderBy: OrderBy | OrderBy[];
    filter: Filter | Filter[];
    limit: number;
    page: number;
    withSubitems: boolean;
}>;

*/
