import { getSubitemsNodeName } from '../_base/adapterBase';
import { toArray } from 'shared-base';
export const rules = {
    base: (request, getParams) => {
        const { withSubitems } = getParams;
        if (withSubitems) {
            request.params.push({
                field: '_expand',
                value: getSubitemsNodeName(request.path),
            });
        }
        return request;
    },
    rest: (request, getParams) => {
        Object.entries(getParams).forEach(([key, value]) => {
            request.params.push({
                field: key,
                value: value,
            });
        });
        return request;
    },
    sort: (request, getParams) => {
        const { orderBy } = getParams;
        const orderByArray = toArray(orderBy);
        const output = orderByArray.reduce((output, order) => {
            if (order.field) {
                output._sort.push(order.field);
                output._order.push(order.order ?? 'asc');
            }
            return output;
        }, {
            _sort: [],
            _order: [],
        });
        if (output._sort.length) {
            request.params.push({
                field: '_sort',
                value: output._sort.join(','),
            });
        }
        if (output._order.length) {
            request.params.push({
                field: '_order',
                value: output._order.join(','),
            });
        }
        return request;
    },
    fullTextSearch: (request, getParams) => {
        const { q } = getParams;
        if (q && q.value) {
            request.params.push({
                field: 'q',
                value: q?.value,
            });
            if (q.fields) {
                request.params.push({
                    field: 'fields',
                    value: q.fields.join(','),
                });
            }
        }
        return request;
    },
    filter: (request, getParams) => {
        const { filter } = getParams;
        const filterArray = toArray(filter);
        filterArray.forEach((filter) => {
            const { field, relation, value } = filter;
            let suffix = '';
            switch (relation) {
                case '==':
                    suffix = '';
                    break;
                case '>=':
                    suffix = '_gte';
                    break;
                case '<=':
                    suffix = '_lte';
                    break;
                case '!=':
                    suffix = '_ne';
                    break;
                case '//g':
                    suffix = '_like';
                    break;
            }
            request.params.push({
                field: `${field}${suffix}`,
                value: value,
            });
        });
        return request;
    },
    paginate: (request, getParams) => {
        const { page, limit } = getParams;
        if (page) {
            request.params.push({
                field: '_page',
                value: page,
            });
        }
        if (limit) {
            request.params.push({
                field: '_limit',
                value: limit,
            });
        }
        return request;
    },
    slice: (request, getParams) => {
        const { slice } = getParams;
        const { start, end } = slice;
        if (start) {
            request.params.push({
                field: '_start',
                value: start,
            });
        }
        if (end) {
            request.params.push({
                field: '_end',
                value: end,
            });
        }
        return request;
    },
};
