import { NodeType } from 'redux-store-generator';
import { RequestStatus, } from '../types';
import { generateIds } from '../utils/ids';
let sequence = 1;
export class RequestBuilder {
    constructor() {
        this.output = {
            requestStatus: RequestStatus.CREATED,
            items: [],
            optimistic: false,
            optimisticPosts: false,
        };
        this.output = {
            ...generateIds(sequence++),
            ...this.output,
        };
    }
    withConnectionType(connectionType) {
        this.output.argsConnectionType = connectionType;
        return this;
    }
    withNodeName(value) {
        this.output.argsNodeName = value;
        return this;
    }
    withMethod(value) {
        this.output.argsApiVerb = value;
        return this;
    }
    withNodeType(value) {
        this.output.argsNodeType = value;
        return this;
    }
    withParams(params) {
        this.output.argsParams = params;
        return this;
    }
    withOptimistic(optimistic) {
        this.output.optimistic = optimistic;
        return this;
    }
    withOptimisticPosts(optimisticPosts) {
        this.output.optimisticPosts = optimisticPosts;
        return this;
    }
    withOriginalAction(value) {
        this.output.originalAction = value;
        this.output.resolve = value.resolve;
        this.output.reject = value.reject;
        this.output.resourceId = value.id;
        this.output.resourceItemId = value.itemId;
        this.output.argsParams = value.payload;
        return this;
    }
    build() {
        if (!this.output.argsParams) {
            delete this.output.argsParams;
        }
        const isCollection = this.output.argsNodeType === NodeType.COLLECTION_NODE ||
            this.output.argsNodeType === NodeType.GROUPED_LIST_NODE;
        const { argsNodeName, resourceId, resourceItemId, argsApiVerb } = this.output;
        switch (argsApiVerb) {
            case 'get':
                this.output.argsPath = `/${argsNodeName}`;
                this.output.argsMethod = 'GET';
                break;
            case 'patch':
                this.output.argsPath = `/${argsNodeName}`;
                if (isCollection) {
                    this.output.argsPath += `/${resourceId}`;
                }
                this.output.argsMethod = 'PATCH';
                break;
            case 'push':
                this.output.argsPath = `/${argsNodeName}`;
                this.output.argsMethod = 'POST';
                break;
            case 'add':
                this.output.argsPath = `/${argsNodeName}`;
                this.output.argsMethod = 'POST';
                break;
            case 'delete':
                this.output.argsPath = `/${argsNodeName}`;
                if (isCollection) {
                    this.output.argsPath += `/${resourceId}`;
                }
                this.output.argsMethod = 'DELETE';
                break;
            case 'pushItem':
                this.output.argsPath = `/${argsNodeName}/${resourceId}`;
                this.output.argsMethod = 'POST';
                break;
            case 'deleteItem':
                this.output.argsPath = `/${argsNodeName}/${resourceId}/items/${resourceItemId}`;
                this.output.argsMethod = 'DELETE';
                break;
            case 'patchItem':
                this.output.argsPath = `/${argsNodeName}/${resourceId}/items/${resourceItemId}`;
                this.output.argsMethod = 'PATCH';
                break;
            case 'getItems':
                this.output.argsPath = `/${argsNodeName}/${resourceId}/items`;
                this.output.argsMethod = 'GET';
                break;
        }
        return this.output;
    }
}
