import globals from './utils/globals';
import { connectedMiddleware } from './middlewares/midConnected';
import { generateConnectedStore } from './store/store';
let DEBUG = false;
export const initReduxConnected = (config, storeBuilder) => {
    const { logger = defaultLogger } = config;
    logger('redux-connected: init');
    storeBuilder.withMiddlewares([connectedMiddleware]);
    storeBuilder.withPostBuildHook((store) => {
        logger('redux-connected: postBuildHook');
        return initConnectedStore(config, store);
    });
};
const initConnectedStore = (config, store) => {
    const { logger = defaultLogger } = config;
    logger('redux-connected: initConnectedStore');
    const state = flatten(store.getState());
    globals.mainStore = store;
    globals.adapters = config.adapters;
    globals.structure = state;
    globals.connectedStore = generateConnectedStore(state, config);
    logger('redux-connected: store is ready', globals.connectedStore);
};
export const getConnectedStore = () => {
    return globals.connectedStore;
};
const defaultLogger = (message, data) => {
    if (!DEBUG) {
        return;
    }
    console.log(message, data);
};
const flatten = (storeState) => {
    const output = {};
    Object.keys(storeState).forEach((appId) => {
        const value = storeState[appId];
        Object.keys(value).forEach((nodeName) => {
            output[nodeName] = value[nodeName];
        });
    });
    return output;
};
