import * as devtools from '../utils/devtoolsBridge';
import globals from '../utils/globals';
import sagas from '../sagas';
import { cleanInitialState, generateInitialState } from './initialState';
import { devtoolsMiddleware } from '../middlewares/midDevtools';
import { gatekeeperMiddleware } from '../middlewares/midGatekeeper';
import { StoreBuilder } from '../builders/StoreBuilder';
import { generateReducersForStore } from 'redux-store-generator'; // prettier-ignore
export const generateConnectedStore = (state, config, middlewares = []) => {
    const connectedStoreBuilder = new StoreBuilder('connected');
    const schema = generateInitialState(state, config);
    connectedStoreBuilder
        .withReducers('connected', {
        ...generateReducersForStore(schema),
    })
        .withReducers('connected', {
        _lastAction,
    })
        .withMiddlewares([gatekeeperMiddleware])
        .withMiddlewares(middlewares)
        .withSagas(sagas.requests.saga, sagas.incoming.saga, sagas.postAction.saga, sagas.houseKeeping.saga);
    const initialState = cleanInitialState(schema);
    connectedStoreBuilder.withInitialState('connected', initialState);
    if (config.enableReduxDevtools) {
        connectedStoreBuilder.withMiddlewares(devtoolsMiddleware);
    }
    if (config.enableReduxDevtools) {
        connectedStoreBuilder.withPostBuildHook((_store) => {
            devtools.sendState(_store.getState());
        });
    }
    globals.connectedStore = connectedStoreBuilder.build();
    return globals.connectedStore;
};
export const generateConnectedStoreEmpty = (state, config) => {
    const connectedStoreBuilder = new StoreBuilder('connected');
    const initialState = generateInitialState(state, config);
    connectedStoreBuilder
        .withInitialState('connected', initialState)
        .withReducers('connected', {
        ...generateReducersForStore(initialState),
    })
        .withReducers('connected', {
        _lastAction,
    });
    return connectedStoreBuilder.build();
};
// https://github.com/reduxjs/redux/issues/580#issuecomment-133188511
export const _lastAction = (_state, action) => {
    return action;
};
