export const parsePath = (path) => {
    const parts = path.split('/');
    const hasItems = parts[3] === 'items';
    return {
        nodeName: parts[1],
        id: parts[2],
        hasItems,
        subItemId: hasItems ? parts[4] : undefined,
    };
};
export const getIndex = (path, data) => {
    let { id, nodeName, subItemId } = parsePath(path);
    if (subItemId) {
        nodeName = getSubitemsNodeName(path);
        id = subItemId;
    }
    const array = data[nodeName];
    return array.findIndex((item) => item.id === id);
};
export const toSingle = (name) => {
    return name.replace(/e?s$/, '');
};
export const getSubitemsNodeName = (path) => {
    const { nodeName } = parsePath(path);
    const single = toSingle(nodeName);
    return `${single}Items`;
};
export const getSubitemsParentFieldName = (path) => {
    const { nodeName } = parsePath(path);
    const single = toSingle(nodeName);
    return `${single}Id`;
};
export const getSubitems = (path, data) => {
    const itemsNodeName = getSubitemsNodeName(path);
    return data[itemsNodeName];
};
export const getSubitemIndex = (path, data) => {
    const { id, nodeName } = parsePath(path);
    const array = data[nodeName];
    return array.findIndex((item) => item.id === id);
};
export const getPathType = (path, data) => {
    const { id, nodeName, hasItems } = parsePath(path);
    const root = data[nodeName];
    if (hasItems) {
        return 'SUBITEM';
    }
    if (id) {
        return 'SINGLE';
    }
    return isQueue(root) || isCollection(root) ? 'COLLECTION' : 'SINGLE';
};
export const isQueue = (data) => {
    return Array.isArray(data);
};
export const isCollection = (data) => {
    if (typeof data !== 'object') {
        return false;
    }
    const firstKey = Object.keys(data)[0];
    const firstValue = data[firstKey];
    return (typeof firstValue === 'object' &&
        firstValue['id'] &&
        firstValue['id'] === firstKey);
};
